#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 17-7-8
# @Author  : JCai
# @desc    :

import os
import random
import django

from utils.util import str_to_date

os.environ['DJANGO_SETTINGS_MODULE'] = 'EBayClub.settings'
django.setup()

from datetime import datetime, timedelta

import pytz
from django.utils import timezone
from info_sync.interface.trading_inter import MyEbay
from info_sync.models.listing_info_models import EbayListing, EbayListingVariations, EBayListingInfoMore
from utils.logger import logStatusBegin, logStatusEnd, logInfo
from info_sync.models.account_models import EBayAccount


from_time = str(datetime.now(pytz.UTC)).split('.')[0].replace(' ', 'T') + 'Z'  # - timedelta(days=3)
to_time = str(datetime.now(pytz.UTC) + timedelta(days=33)).split('.')[0].replace(' ', 'T') + 'Z'

info_path = '/home/ec2e/ebay_listing_info'  # ec2e
bak_path = '/home/ec2e/bak'
#info file: (store)_type_datetime.csv
base_file_name = 'base'
more_file_name = 'more'
log_file_name = '{base_dir}/log/{date}_{random}.log'.format(base_dir=info_path,
                        date=timezone.now().strftime('%Y-%m-%d'), random=random.randint(100, 999))


def wki_log(msg):
    with open(log_file_name, 'a+') as f:  # 没必要对log加锁
        f.write(timezone.now().strftime('%Y-%m-%d %H:%M:%S')+', ')
        f.write(msg)
        f.write('\n')


class MyException(Exception):
    def __init__(self, message):
        Exception.__init__(self)
        self.message = message


# ** read **
def base_info_sync_to_file(store):
    store_name = store.get('name', '')
    result = False
    print 'do base_info_sync_to_file() on: '+store_name+'\n'
    base_path = '{base_dir}/({store})_{type}_{datetime}.csv'.format(base_dir=info_path,
                                                                store=store_name,
                                                                type=base_file_name,
                                                                datetime=timezone.now().strftime('%Y-%m-%d'))
    store_name = store.get('name', '')
    token = store.get('Token', '')
    if not token or len(token) <= 0:
        wki_log('account error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        return result, ''
    store_id = store.get('account_id', '')
    if not store_id or store_id <= 0:
        wki_log('account error:\t%s\n\t%s' % (store_name, 'Store ID is Empty!'))
        return result, ''
    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    call_data = {
        'EndTimeFrom': from_time,
        'IncludeWatchCount': True,
        'EndTimeTo': to_time,
        'IncludeVariations': True,
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_base_selector_keys(),
        'Pagination': {'EntriesPerPage': 200},
        'GranularityLevel': 'Fine'
    }
    page = 1
    while True:
        try:
            call_data['Pagination']['PageNumber'] = page
            status, message, listing_items, is_next_get = my_ebay.get_seller_list(call_data)
            if not status:
                # retry
                status, message, listing_items, is_next_get = my_ebay.get_seller_list(call_data)
                if not status:
                    wki_log('got error(get_seller_list): %s\t%s' % (store_name, message))
                    os.system('rm ' + base_path)
                    return result, ''
                else:
                    wki_log('base write log(retry):%s, page:%d, count:%d' % (store_name, page, len(listing_items)))
            else:
                wki_log('base write log:%s, page:%d, count:%d'%(store_name,page,len(listing_items)))
            page += 1
            if type(listing_items) == type({}):
                listing_items = [listing_items]
            base_info_save_to_file(listing_items, store_name, base_path)
            '''
            for each in listing_items:
                item_id = each.get('ItemID', '')
                open_date = str_to_date(each.get('ListingDetails', {}).get('StartTime', ''))
                variations = each.get('Variations', {}).get('Variation', [])
                if type(variations) == type({}):  # if only have a variations  ,the resutl is a dict
                    variations = [variations]
                variation_list, is_variation_refresh = info_variation_change_to_sql(item_id, variations, open_date)
                info_base_change_to_sql(item_id, each, store_id, len(variation_list), open_date)
                base_info_save_to_file(base_path)
                variations_info_save_to_file(item_id,variation_list,variations_path)
            '''
            if not is_next_get:
                break
        except Exception, e:
            wki_log('unknown error in base_info:\t%s\n\t%s' % (store_name, str(e)))
            os.system('rm ' + base_path)
            return result, ''
    result = True
    return result, base_path


def listing_base_selector_keys():
    return ['ItemArray.Item.ItemID',
            'ItemArray.Item.Site',
            'ItemArray.Item.SellingStatus.ListingStatus',
            'ItemArray.Item.ListingType',
            'ItemArray.Item.SKU',
            'ItemArray.Item.Title',
            'ItemArray.Item.SubTitle',
            'ItemArray.Item.Currency',
            'ItemArray.Item.SellingStatus.CurrentPrice',
            'ItemArray.Item.StartPrice',
            'ItemArray.Item.ReservePrice',
            'ItemArray.Item.BuyItNowPrice',
            'ItemArray.Item.ListingDuration',
            'ItemArray.Item.Quantity',
            'ItemArray.Item.SellingStatus.QuantitySold',
            'ItemArray.Item.ListingDetails.StartTime',
            'ItemArray.Item.ListingDetails.EndTime',
            'ItemArray.Item.Variations',
            'ItemArray.Item.Country',
            'ItemArray.Item.Location',
            'ItemArray.Item.DispatchTimeMax',
            'PaginationResult']


def base_info_save_to_file(listing_items, store_name, file_path):
    # 每个线程写自己的文件，而且是写完再读，暂时不必加锁
    try:
        with open(file_path, 'a+') as base_info_file:
            for item in listing_items:
                item_id = item.get('ItemID', '')
                ListingStatus = item.get('SellingStatus', {}).get('ListingStatus', '')
                WatchCount = int(item.get('WatchCount', 0))
                Site = item.get('Site', '')
                ListingType = item.get('ListingType', '')
                seller_sku = item.get('SKU', '')
                Title = item.get('Title', '')
                currencyID = item.get('Currency', '')
                listing_price = float(item.get('SellingStatus', {}).get('CurrentPrice', {}).get('value', '0'))
                ListingDuration = item.get('ListingDuration', '')
                quantity = int(item.get('Quantity', '0'))
                QuantitySold = int(item.get('SellingStatus', {}).get('QuantitySold', '0'))
                Country = item.get('Country', '')
                Location = item.get('Location', '')
                start_date = str_to_date(item.get('ListingDetails', {}).get('StartTime', ''))
                end_date = item.get('ListingDetails', {}).get('EndTime', '')
                DispatchTime = item.get('DispatchTimeMax', '')
                variations = item.get('Variations', {})  # .get('Variation', [])
                base_info_file.write('{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n'.format(
                    item_id,ListingStatus,WatchCount,Site,ListingType,seller_sku,Title,currencyID,listing_price,
                    ListingDuration,quantity,QuantitySold,Country,Location,start_date,end_date,DispatchTime,variations))
            base_info_file.flush()
        result = True  # !!!
        '''except MyException, mye:
            fail = fail + 1
            message = message + '<br>' + _spu + ':  ' + mye.message
            continue
        '''
    except Exception, e:
        result = False
        wki_log('save error:\t%s\n\t%s' % (store_name, str(e)))
    return result


def more_info_sync_to_file(base_path, store):
    store_name = store.get('name', '')
    curr_dt = timezone.now().strftime('%Y-%m-%d')
    print 'do more_info_sync_to_file() on: ' + store_name + '\n'
    more_path = '{base_dir}/({store})_{type}_{datetime}.csv'.format(base_dir=info_path,
                                                                store=store_name,
                                                                type=more_file_name,
                                                                datetime=curr_dt)
    token = store.get('Token', '')
    if not token or len(token) <= 0:
        wki_log('account error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        return

    call_data = {
        'IncludeVariations': True,
        'IncludeItemSpecifics': True,
        'GranularityLevel': 'Fine',
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_more_selector_keys(),
    }
    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    number = 1
    with open(base_path, 'r') as base_info_file:
        lines = base_info_file.readlines()
        for l in lines:
            base_info = l.split('\t')
            item_id = base_info[0]
            try:
                call_data['ItemID'] = item_id
                status, message, listing_items = my_ebay.get_item(call_data)
                number += 1
                if not status:
                    wki_log('get error(get_item):\t%s:%s\n\t%s' % (store_name, item_id, message))
                    continue
                more_info_save_to_file(item_id, listing_items, store_name, more_path)
            except Exception, e:
                wki_log('unknown error in more_info:\t%s:%s\n\t%s' % (store_name, item_id, str(e)))
    wki_log('more write log:%s, count:%d' % (store_name, number))
    # 空间不足,压缩后删除
    os.chdir(info_path)
    os.system('tar -czvf ' + store_name + '.tar.gz "(' + store_name + ')_base_' + curr_dt + '.csv" "('
              + store_name + ')_more_' + curr_dt + '.csv"')
    os.system('rm "(' + store_name + ')_base_' + curr_dt + '.csv"')
    os.system('rm "(' + store_name + ')_more_' + curr_dt + '.csv"')
    # 通知过来抓取
    #import urllib2
    #req = urllib2.Request('http://122.5.32.82:8092/notify/syncfile/?filename=' + store_name + '.tar.gz')
    #urllib2.urlopen(req)


def listing_more_selector_keys():
    return ['Item.ItemID',
            'Item.PrimaryCategory',
            'Item.SecondaryCategory',
            'Item.Storefront',
            'Item.ConditionID',
            'Item.Description',
            'Item.PictureDetails.PictureURL',
            'Item.DispatchTimeMax',
            'Item.ShippingDetails.ShippingServiceOptions',
            'Item.ShippingDetails.InternationalShippingServiceOption',
            'Item.ShippingDetails.ExcludeShipToLocation',
            'Item.ReturnPolicy',
            'Item.BuyerRequirementDetails',
            'Item.ItemSpecifics.NameValueList',
            'Item.Variations',
            'Item.PaymentMethods',
            'Item.PayPalEmailAddress',
            'Item.PaymentInstructions',
            'Item.PostalCode',
            'Item.ProductListingDetails',
            'Item.ListingDetails',
            'Item.BestOfferDetails']


def more_info_save_to_file(item_id, item_info, store_name, file_path):
    try:
        description = ''  # str(item_info.get('Description', '')).replace('\t', '')
        image_list = str(item_info.get('PictureDetails', {}).get('PictureURL', []))
        dispatch_time_max = int(item_info.get('DispatchTimeMax', '0'))
        shipping_domestic = str(item_info.get('ShippingDetails', {}).get('ShippingServiceOptions', []))
        shipping_international = str(item_info.get('ShippingDetails', {}).get('InternationalShippingServiceOption', []))
        exclude_ship_to_location = str(item_info.get('ShippingDetails', {}).get('ExcludeShipToLocation', []))
        return_policy = str(item_info.get('ReturnPolicy', []))
        buyer_requirement = str(item_info.get('BuyerRequirementDetails', []))
        category_specifics = str(item_info.get('ItemSpecifics', {}).get('NameValueList', []))
        payment_methods_list = item_info.get('PaymentMethods', [])
        if type(payment_methods_list) != type([]):
            payment_methods = str([payment_methods_list])
        else:
            payment_methods = str(payment_methods_list)
        paypal_email_address = item_info.get('PayPalEmailAddress', '')
        primary_category_id = item_info.get('PrimaryCategory', {}).get('CategoryID', '')
        secondary_category_id = item_info.get('SecondaryCategory', {}).get('CategoryID', '')
        store_category_id = item_info.get('Storefront', {}).get('StoreCategoryID', '')
        store_category_2id = item_info.get('Storefront', {}).get('StoreCategory2ID', '')
        primary_category_name = item_info.get('PrimaryCategory', {}).get('CategoryName', '')
        secondary_category_name = item_info.get('SecondaryCategory', {}).get('CategoryName', '')
        condition_id = item_info.get('ConditionID', '')
        ProductListingDetails = item_info.get('ProductListingDetails', '')
        Variations = item_info.get('Variations', '')
        PostalCode = item_info.get('PostalCode', '')
        best_offer_enabled = item_info.get('BestOfferDetails', {}).get('BestOfferEnabled', 0)
        if best_offer_enabled and best_offer_enabled == 'true':
            best_offer_enabled = 1
        else:
            best_offer_enabled = 0
        listing_details = item_info.get('ListingDetails', {})
        best_offer_auto_accept_price = listing_details.get('BestOfferAutoAcceptPrice', {}).get('value', 0)
        minimum_best_offer_price = listing_details.get('MinimumBestOfferPrice', {}).get('value', 0)
        # 每个线程写自己的文件，而且是写完再读，暂时不必加锁
        with open(file_path, 'a+') as more_info_file:
            more_info_file.write('{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n'.format(
                item_id,description,image_list,dispatch_time_max,shipping_domestic,shipping_international,exclude_ship_to_location,
                return_policy,buyer_requirement,category_specifics,payment_methods,paypal_email_address,primary_category_id,
                secondary_category_id,store_category_id,store_category_2id,primary_category_name,secondary_category_name,
                condition_id,ProductListingDetails,Variations,PostalCode,best_offer_enabled,best_offer_auto_accept_price,minimum_best_offer_price))
            more_info_file.flush()
        result = True  # !!!
    except Exception,e:
        result = False
        wki_log('save error:\t%s\n\t%s' % (store_name, str(e)))
    return result


# ** write **
def info_base_save_to_sql(store_name, path):
    store = EBayAccount.objects.filter(name=store_name).first()
    with open(path, 'r') as base_info_file:
        lines = base_info_file.readlines()
        for l in lines:
            base_info = l.split('\t')
            item_id = base_info[0]
            ListingStatus = base_info[1]
            WatchCount = base_info[2]
            Site = base_info[3]
            ListingType = base_info[4]
            seller_sku = base_info[5]
            Title = base_info[6]
            currencyID = base_info[7]
            listing_price = base_info[8]
            ListingDuration = base_info[9]
            quantity = base_info[10]
            QuantitySold = base_info[11]
            Country = base_info[12]
            Location = base_info[13]
            open_date = base_info[14]
            end_date = base_info[15]

            no_single_day = 0
            listing_end_time = datetime.strptime(timezone.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
            if QuantitySold == 0:
                start_time = datetime.strptime(open_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
                no_single_day = (listing_end_time - start_time).days

            _update_info = {
                'Country': Country,
                'Site': Site,
                'ListingDuration': ListingDuration,
                'quantity': quantity,
                'listing_price': listing_price,
                'currencyID': currencyID,
                'ListingStatus': ListingStatus,
                'ListingType': ListingType,
                'QuantitySold': QuantitySold,
                'Title': Title,
                'seller_sku': seller_sku,
                'WatchCount': WatchCount,
                #'image': PictureDetails,  # move to "more_info"
                #'CategoryID': PrimaryCategory ,  # move to "more_info"
                #'CategoryName': PrimaryCategory,  # move to "more_info"
                #'StoreCategoryID': Storefront,  # move to "more_info"
                'ShippingServiceCost': 0.0,
                'Location': Location,
                #'warehouse': warehouse
            }
            #warehouse = get_ware_str(_item.get('Location', ''))
            try:
                variations = eval(base_info[16][:-1])
            except:
                variations = {}
            #base_info['write_date'] = timezone.now().strftime('%Y-%m-%dT%H:%M:%SZ')

            variation_count = 0
            if variations:
                variation_count = info_variation_save_to_sql(item_id, variations, ListingStatus, open_date)
                _update_info['ListingType'] = 'Variation'

            base_listings = EbayListing.objects.filter(ItemID=item_id)
            base_listing = base_listings.first()
            if base_listing:
                if QuantitySold != 0:
                    no_single_day = int(base_listing.no_single_day)
                    start_time = datetime.strptime(base_listing.write_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
                    day_temp = (listing_end_time - start_time).days
                    if day_temp > 0:
                        if int(base_listing.QuantitySold) == QuantitySold:
                            no_single_day += day_temp
                        else:
                            no_single_day = 0
                    #_update_info['no_single_day'] = no_single_day
                base_listings.update(
                    open_date=open_date,
                    end_date=end_date,
                    #refresh=1,
                    variation=variation_count,
                    no_single_day=no_single_day,
                    write_date=timezone.now(),
                    **_update_info)

            else:
                ebay_listing = EbayListing(
                    ItemID=item_id,
                    ebay_account_id=store.account_id,
                    open_date=open_date,
                    end_date=end_date,
                    #refresh=1,
                    no_single_day=no_single_day,
                    variation=variation_count,
                    create_date=timezone.now(),
                    write_date=timezone.now(),
                    **_update_info
                )
                ebay_listing.save()


def info_variation_save_to_sql(item_id, variations, ListingStatus, open_date):
    #EbayListingVariations.objects.filter(ItemID=item_id).update(refresh=-1)
    for variation in variations.get('Variation',[]):
        seller_sku = variation.get('SKU', '')
        variation_info = {}
        variation_listings = EbayListingVariations.objects.filter(ItemID=item_id, seller_sku=seller_sku)
        variation_listing = variation_listings.first()
        if ListingStatus != 'Active':
            if variation_listing:
                if -1 != int(variation_listing.refresh):
                    variation_info['refresh'] = -1
                variation_info['write_date'] = timezone.now().strftime('%Y-%m-%dT%H:%M:%SZ')
                variation_info['seller_sku'] = seller_sku
                variation_listings.update(**variation_info)
            continue

        VariationSpecifics = variation.get('VariationSpecifics', {}).get('NameValueList', [])
        if type(VariationSpecifics) == type({}):
            VariationSpecifics = [VariationSpecifics]

        CurrentPrice = float(variation.get('StartPrice', {}).get('value', '0.0'))
        Quantity = int(variation.get('Quantity', '0'))
        QuantitySold = int(variation.get('SellingStatus', {}).get('QuantitySold', '0'))

        now_time = datetime.strptime(timezone.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
        no_single_day = 0
        if QuantitySold == 0:
            #start_time = datetime.strptime(open_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
            start_time = datetime.strptime(open_date.split('+')[0], '%Y-%m-%d %H:%M:%S')  # 2017-07-06 01:34:50+00:00
            no_single_day = (now_time - start_time).days

        if variation_listing:
            if QuantitySold != 0:
                no_single_day = int(variation_listing.no_single_day)
                start_time = datetime.strptime(variation_listing.write_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
                day_temp = (now_time - start_time).days
                if day_temp > 0:
                    if int(variation_listing.QuantitySold) == QuantitySold:
                        no_single_day += day_temp
                    else:
                        no_single_day = 0
        else:
            variation_info['create_date'] = timezone.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        variation_info['CurrentPrice'] = CurrentPrice
        variation_info['Quantity'] = Quantity
        variation_info['QuantitySold'] = QuantitySold
        variation_info['VariationSpecifics'] = VariationSpecifics
        variation_info['no_single_day'] = no_single_day
        variation_info['write_date'] = timezone.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        variation_info['seller_sku'] = seller_sku
        variation_info['refresh'] = 1

        if variation_listings:  # update
            variation_listings.update(**variation_info)
        else:  # save
            ebay_listing_variation = EbayListingVariations(ItemID=item_id,**variation_info)
            ebay_listing_variation.save()
    return 0


def get_more_from_file(file,item_count):
    eof = False
    more_info = []
    c = ' '
    local_count = 0
    temp_str = ''
    try:
        while len(c) > 0:
            c = file.read(1)
            if c == '\t':
                more_info.append(temp_str)
                temp_str = ''
                local_count += 1
                if local_count == item_count-1:
                    more_info.append(file.readline())
                    return eof, more_info
                    #if file.read(1) == '\n':
                    #    return eof, more_info
                    #else:
                    #    local_count = 0
                    #    continue
            else:
                temp_str = temp_str + c
    except:
        return True, []
    eof = True
    return eof, more_info


def info_more_save_to_sql(store_name, path):
    with open(path, 'r') as more_info_file:
        #lines = more_info_file.readlines()
        #for l in lines:
        eof = False
        while not eof:
            eof, more_info = get_more_from_file(more_info_file,19)
            #more_info = l.split('\t')
            if not more_info:
                continue
            item_id = more_info[0]
            _update_info = {
                'description': more_info[1],
                'image_list': more_info[2],
                'dispatch_time_max': more_info[3],
                'shipping_domestic': more_info[4],
                'shipping_international': more_info[5],
                'exclude_ship_to_location': more_info[6],
                'return_policy': more_info[7],
                'buyer_requirement': more_info[8],
                'category_specifics': more_info[9],
                'payment_methods': more_info[10],
                'paypal_email_address': more_info[11],
                'primary_category_id': more_info[12],
                'secondary_category_id': more_info[13],
                'store_category_id': more_info[14],
                'store_category_2id': more_info[15],
                'primary_category_name': more_info[16],
                'secondary_category_name': more_info[17],
                'condition_id': more_info[18][:-1]
            }
            more_listings = EBayListingInfoMore.objects.filter(ItemID=item_id)
            if more_listings:
                more_listings.update(**_update_info)
            else:
                EBayListingInfoMore.objects.create(ItemID=item_id, **_update_info)
    return


# ** main **
def ebay_listing_sync_to_file(store):  # read
    import time
    wki_log(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
    # logStatusBegin('Listing Sync Begin:%s' % store.get('name', ''))
    # 获取并保存基本Listing信息
    ret, path = base_info_sync_to_file(store)
    # 获取并保存更多Listing信息
    if ret:
        more_info_sync_to_file(path, store)
    else:
        print 'base info fail,not read more'
        wki_log('base info fail,not read more')
    # logStatusEnd('Listing Sync End:%s' % store.get('name', ''))

    #os.system('cmd')
    #'tar -czvf all.tar.gz *.csv'
    #'rsync -avzP  ec2e@54.196.103.47:~/pan/asd.csv.tgz  ./'
    #'cd ...'
    #'tar -xzvf all.tar.gz'
    wki_log(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))


def ebay_listing_file_to_SQL():  # write
    #curr_dt = timezone.now().strftime('%Y-%m-%d')
    #os.chdir(info_path)
    #print 'tar -czvf ' + 'ebiz1225' + '.tar.gz "(' + 'ebiz1225' + ')_base_' + '2017-07-31' + '.csv" "('+'ebiz1225' + ')_more_' + '2017-07-31' + '.csv"'
    #os.system('tar -czvf ' + 'ebiz1225' + '.tar.gz "(' + 'ebiz1225' + ')_base_' + '2017-07-31' + '.csv" "('
    #          + 'ebiz1225' + ')_more_' + '2017-07-31' + '.csv"')
    file_list = os.listdir(info_path)
    for i in range(0, len(file_list)):
        path = os.path.join(info_path, file_list[i])
        if os.path.isfile(path):
            store_name = file_list[i].split(')_')[0][1:]
            type = file_list[i].split(')_')[1].split('_')[0]
            if type == 'base':
                info_base_save_to_sql(store_name, path)
            elif type == 'more':
                info_more_save_to_sql(store_name, path)
